MezUI = LibStub("AceAddon-3.0"):NewAddon("MezUI", "AceConsole-3.0", "AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("MezUI")
local mezdb
local LSM = LibStub("LibSharedMedia-3.0")

local E, L, V, P, G = unpack(ElvUI);

-- Player Frame
local poFrame = CreateFrame("Frame", "poFrame", UIParent)
local poTexture = poFrame:CreateTexture(nil,"BACKGROUND")
--[[
local poScrollFrame = poScrollFrame or CreateFrame("ScrollFrame", "poScrollFrame", poFrame);
local poScrollChild = poScrollChild or CreateFrame("Frame", "poScrollChild");
local poChildTexure = poScrollChild:CreateTexture(nil,"BACKGROUND")
]]

-- Target Frame
local toFrame = CreateFrame("Frame", "toFrame", UIParent)
local toTexture = toFrame:CreateTexture(nil,"BACKGROUND")

-- Player Icon
local piFrame = CreateFrame("Frame", "piFrame", UIParent)
local piTexture = piFrame:CreateTexture(nil,"OVERLAY")

-- Target Alliance Icon
local taiFrame = CreateFrame("Frame", "taiFrame", UIParent)
local taiTexture = taiFrame:CreateTexture(nil,"OVERLAY")

-- Target Horde Icon
local thiFrame = CreateFrame("Frame", "thiFrame", UIParent)
local thiTexture = thiFrame:CreateTexture(nil,"OVERLAY")

---/// Interface Options \\\---
---=========================---
local defaults = {
        -- General Overlay Variables
		overlayEnabled = true,
		overlayLock = true,
		
		-- Faction Icon Variables
		iconPlayerLock = true,
		iconTargAllianceLock = true,
		iconTargHordeLock = true,
		
		iconPlayerEnabled = true,
		iconPlayerStrata = "MEDIUM",
		iconPlayerScale = 77,
		iconPlayerAlpha = 1,
		iconPlayerAllianceTexture = "muiac",
		iconPlayerHordeTexture = "muihc",
		pIconTexture = iconPlayerAllianceTexture,
		iconPlayerXpos = (0.5 * GetScreenWidth()),
		iconPlayerYpos = (0.5 * GetScreenHeight()),
		
		iconTargAllianceEnabled = true,
		iconTargAllianceStrata = "MEDIUM",
		iconTargAllianceScale = 77,
		iconTargAllianceAlpha = 1,
		iconTargAllianceTexture = "muiac",
		iconTargAllianceXpos = (0.5 * GetScreenWidth()),
		iconTargAllianceYpos = (0.5 * GetScreenHeight()),
		
		iconTargHordeEnabled = true,
		iconTargHordeStrata = "MEDIUM",
		iconTargHordeScale = 77,
		iconTargHordeAlpha = 1,
		iconTargHordeTexture = "muihc",
		iconTargHordeXpos = (0.5 * GetScreenWidth()),
		iconTargHordeYpos = (0.5 * GetScreenHeight()),
		
		-- Player Overlay Variables
		overlayPlayerHeight = 40,
		overlayPlayerWidth = 240,
		bgPlayerAlpha = 0.7,
		bgPlayerAllianceTexture = "muibg",
		bgPlayerHordeTexture = "muibg",
		bgPlayerTexture = bgPlayerAllianceTexture,
		overlayPlayerXpos = (0.5 * GetScreenWidth()),
		overlayPlayerYpos = (0.5 * GetScreenHeight()),
		poChildLock = true,
		
		-- Target Overlay Variables
		overlayTargHeight = 40,
		overlayTargWidth = 240,
		bgTargAlpha = 0.7,
		bgTargAllianceTexture = "muibg",
		bgTargHordeTexture = "muibg",
		bgTargNeutralTexture = "muibg",
		bgTargTexture = bgTargNeutralTexture,
		overlayTargXpos = (0.5 * GetScreenWidth()),
		overlayTargYpos = (0.5 * GetScreenHeight()),
}

local options = {
    name = "MezUI",
    handler = MezUI,
    type = "group",
    args = {
		muiHealthbarOverlays = {
			name = "Healthbar Overlays",
			order = 1,
			type = "group",
			childGroups = "tab",
			args = {
				muiGenDesc1 = {
					order = 0,
					type = "description",
					fontSize = "medium",
					name = "Add flare to the UI by adding a texture to your healthbar that can be theme'd to your liking for Alliance, Horde or Neutral faction units. Maybe horde factioned units have scratchy wild healthbars while alliance are clean and ordered, and neutral are blank? You decide.",
				},
				muiGenBlank1 = {
					order = 1,
					type = "description",
					name = " ",
				},
				muiGenDesc2 = {
					order = 2,
					type = "description",
					fontSize = "medium",
					name = "The healthbar overlays can be moved (by unlocking) and resized manually with the sliders OR if you happen to use ElvUI, one click of the button 'Set to ElvUI Unitframes' will automatically set the size and positioning of the overlays to your player and target healthbars.",
				},
				muiGenBlank2 = {
					order = 3,
					type = "description",
					name = " ",
				},
				muiGeneralOverlay = {
					order = 4,
					type = "group",
					name = L["General"],
					inline = true,
					args = {
						overlayEnabled = {
							order = 1,
							type = "toggle",
							name = L["Enabled"],
							desc = "Enable/Disable overlays",
							get = function(info) return mezdb.overlayEnabled end,
							set = function(info,value) mezdb.overlayEnabled = value 
								muiUpdateBG()
							end,
						},
						overlayLock = {
							order = 2,
							type = "toggle",
							name = L["Healthbar Overlays Locked"],
							desc = "Lock/unlock mouse drag movability",
							get = function(info) return mezdb.overlayLock end,
							set = function(info,value) mezdb.overlayLock = value; 
								muiOverlayLockToggle() 
							end,
						},
						setElvUIPos = {
							order = 3,
							type = "execute",
							name = L["Set to ElvUI Unitframes"],
							desc = "Automatically set to the size and location of ElvUI player and target unitframes if installed",
							func = function()
								muiElvUIConfig()
							end,
						},
					},
				},
				muiPlayerOverlay = {
					order = 5,
					type = "group",
					name = L["Player Overlay"],
					desc = L["Place over player health bar for best results"],
					args = {
						overlayPlayerHeight = {
							order = 4,
							type = "range",
							name = L["Height"],
							desc = "Set to the height of your player healthbar",
							min = 10, max = 500, step = 1,
							get = function(info) return mezdb.overlayPlayerHeight end,
							set = function(info,value) mezdb.overlayPlayerHeight = value 
								poFrame:SetSize(mezdb.overlayPlayerWidth,mezdb.overlayPlayerHeight)
							end,
						},
						overlayPlayerWidth = {
							order = 5,
							type = "range",
							name = L["Width"],
							desc = "Set to the width of your player healthbar",
							min = 50, max = 1000, step = 1,
							get = function(info) return mezdb.overlayPlayerWidth end,
							set = function(info,value) mezdb.overlayPlayerWidth = value 
								poFrame:SetSize(mezdb.overlayPlayerWidth,mezdb.overlayPlayerHeight)
							end,
						},
						bgPlayerAlpha = {
							order = 6,
							type = "range",
							name = L["Background Alpha"],
							desc = "Set desired transparency of texture",
							min = 0.1, max = 1, step = 0.1,
							get = function(info) return mezdb.bgPlayerAlpha end,
							set = function(info,value) mezdb.bgPlayerAlpha = value 
								poTexture:SetAlpha(mezdb.bgPlayerAlpha)
							end,
						},
						bgPlayerAllianceTexture = AceGUIWidgetLSMlists and {
							order = 7,
							type = "select",
							name = L["Select Alliance Background"],
							desc = L["Select the background texture for player overlay"],
							dialogControl = 'LSM30_Statusbar',
							values = AceGUIWidgetLSMlists.statusbar,
							--arg = { "skin", "background" },
							get = function() return mezdb.bgPlayerAllianceTexture end,
							set = function(self,key)
								mezdb.bgPlayerAllianceTexture = key;
								muiUpdateBG()
							end,
						},
						bgPlayerHordeTexture = AceGUIWidgetLSMlists and {
							order = 7,
							type = "select",
							name = L["Select Horde Background"],
							desc = L["Select the background texture for player overlay"],
							dialogControl = 'LSM30_Statusbar',
							values = AceGUIWidgetLSMlists.statusbar,
							--arg = { "skin", "background" },
							get = function() return mezdb.bgPlayerHordeTexture end,
							set = function(self,key)
								mezdb.bgPlayerHordeTexture = key;
								muiUpdateBG()
							end,
						},
						blank4 = {
							type = "description",
							order = 10,
							name = " ",
						},
						--[[
						poChildLock = {
							order = 11,
							type = "toggle",
							name = L["Lock Embedded Faction Display"],
							get = function(info) return mezdb.poChildLock end,
							set = function(info,value) mezdb.poChildLock = value;
								muipoChildLockToggle()
							end,
						},
						]]
						-- Player faction child texture enable toggle
						-- Player faction child texture alpha
						-- Player faction child texture Select dropdown
					},
				},
				muiTargetOverlay = {
					order = 6,
					type = "group",
					name = L["Target Overlay"],
					desc = L["Place over target health bar for best results"],
					args = {
						overlayTargHeight = {
							order = 4,
							type = "range",
							name = L["Height"],
							desc = "Set to the height of your target healthbar",
							min = 10, max = 500, step = 1,
							get = function(info) return mezdb.overlayTargHeight end,
							set = function(info,value) mezdb.overlayTargHeight = value 
								toFrame:SetSize(mezdb.overlayTargWidth,mezdb.overlayTargHeight)
							end,
						},
						overlayTargWidth = {
							order = 5,
							type = "range",
							name = L["Width"],
							desc = "Set to the width of your target healthbar",
							min = 50, max = 1000, step = 1,
							get = function(info) return mezdb.overlayTargWidth end,
							set = function(info,value) mezdb.overlayTargWidth = value 
								toFrame:SetSize(mezdb.overlayTargWidth,mezdb.overlayTargHeight)
							end,
						},
						bgTargAlpha = {
							order = 6,
							type = "range",
							name = L["Background Alpha"],
							desc = "Set desired transparency of texture",
							min = 0.1, max = 1, step = 0.1,
							get = function(info) return mezdb.bgTargAlpha end,
							set = function(info,value) mezdb.bgTargAlpha = value 
								toTexture:SetAlpha(mezdb.bgTargAlpha)
							end,
						},
						bgTargAllianceTexture = AceGUIWidgetLSMlists and {
							order = 7,
							type = "select",
							name = L["Select Alliance Background"],
							desc = L["Select the background texture for alliance overlay"],
							dialogControl = 'LSM30_Statusbar',
							values = AceGUIWidgetLSMlists.statusbar,
							--arg = { "skin", "background" },
							get = function() return mezdb.bgTargAllianceTexture end,
							set = function(self,key)
								mezdb.bgTargAllianceTexture = key;
								muiUpdateBG()
							end,
						},
						bgTargHordeTexture = AceGUIWidgetLSMlists and {
							order = 8,
							type = "select",
							name = L["Select Horde Background"],
							desc = L["Select the background texture for horde overlay"],
							dialogControl = 'LSM30_Statusbar',
							values = AceGUIWidgetLSMlists.statusbar,
							--arg = { "skin", "background" },
							get = function() return mezdb.bgTargHordeTexture end,
							set = function(self,key)
								mezdb.bgTargHordeTexture = key;
								muiUpdateBG()
							end,
						},
						bgTargNeutralTexture = AceGUIWidgetLSMlists and {
							order = 9,
							type = "select",
							name = L["Select Neutral Background"],
							desc = L["Select the background texture for neutral overlay"],
							dialogControl = 'LSM30_Statusbar',
							values = AceGUIWidgetLSMlists.statusbar,
							--arg = { "skin", "background" },
							get = function() return mezdb.bgTargNeutralTexture end,
							set = function(self,key)
								mezdb.bgTargNeutralTexture = key;
								muiUpdateBG()
							end,
						},
					},
				},
			},
		},
		muiFactionIcons = {
			name = "Faction Icons",
			order = 2,
			type = "group",
			childGroups = "tab",
			args = {
				muiFIDesc1 = {
					order = 0,
					type = "description",
					fontSize = "medium",
					name = "A floating icon for both the player and target that will clearly display faction. Just another way to add themed flare to your UI, based on unit faction",
				},
				muiFIBlank1 = {
					order = 1,
					type = "description",
					name = " ",
				},
				muiPlayerIcon = {
					order = 3,
					type = "group",
					name = L["Player Faction Icon"],
					args = {
						iconPlayerEnabled = {
							order = 1,
							type = "toggle",
							name = L["Enabled"],
							desc = "Enable/Disable player faction icon",
							get = function(info) return mezdb.iconPlayerEnabled end,
							set = function(info,value) mezdb.iconPlayerEnabled = value
								muiUpdatePlayerIcon()
							end,
						},
						iconPlayerLock = {
							order = 2,
							type = "toggle",
							name = L["Locked"],
							desc = "Lock/unlock mouse drag movability",
							get = function(info) return mezdb.iconPlayerLock end,
							set = function(info,value) mezdb.iconPlayerLock = value;
								muiIconLockToggle()
							end,
						},
						blank1 = {
							type = "description",
							order = 3,
							name = " ",
						},
						iconPlayerScale = {
							order = 4,
							type = "range",
							name = L["Icon Scale"],
							desc = "Set scale of icon",
							min = 30, max = 512, step = 1,
							get = function(info) return mezdb.iconPlayerScale end,
							set = function(info,value) mezdb.iconPlayerScale = value 
								piFrame:SetSize(mezdb.iconPlayerScale,mezdb.iconPlayerScale)
							end,
						},
						iconPlayerAlpha = {
							order = 5,
							type = "range",
							name = L["Icon Alpha"],
							desc = "Set desired transparency of icon",
							min = 0.1, max = 1, step = 0.1,
							get = function(info) return mezdb.iconPlayerAlpha end,
							set = function(info,value) mezdb.iconPlayerAlpha = value 
								piTexture:SetAlpha(mezdb.iconPlayerAlpha)
							end,
						},
						iconPlayerStrata = {
							order = 6,
							type = "select",
							name = L["Strata"],
							desc = "Set if icon is in foreground or background etc",
							values = {
								['BACKGROUND'] = L["BACKGROUND"],
								['LOW'] = L["LOW"],
								['MEDIUM'] = L["MEDIUM"],
								['HIGH'] = L["HIGH"],
							},
							get = function(info) return mezdb.iconPlayerStrata end,
							set = function(info,value) mezdb.iconPlayerStrata = value; 
								piFrame:SetFrameStrata(mezdb.iconPlayerStrata)
							end,
						},
					},
				},
				muiTargetIcon = {
					order = 4,
					type = "group",
					name = L["Target Faction Icons"],
					args = {
						muiTargetA = {
							order = 1,
							type = "group",
							inline = true,
							name = "Alliance",
							args = {
								iconTargAllianceEnabled = {
									order = 1,
									type = "toggle",
									name = L["Enabled"],
									desc = "Enable/Disable target alliance icon",
									get = function(info) return mezdb.iconTargAllianceEnabled end,
									set = function(info,value) mezdb.iconTargAllianceEnabled = value 
										muiUpdateTargetIcons()
									end,
								},
								iconTargAllianceLock = {
									order = 2,
									type = "toggle",
									name = L["Locked"],
									desc = "Lock/unlock mouse drag movability",
									get = function(info) return mezdb.iconTargAllianceLock end,
									set = function(info,value) mezdb.iconTargAllianceLock = value;
										muiIconLockToggle()
									end,
								},
								blank2 = {
									type = "description",
									order = 3,
									name = " ",
								},
								iconTargAllianceScale = {
									order = 4,
									type = "range",
									name = L["Icon Scale"],
									desc = "Set scale of icon",
									min = 30, max = 512, step = 1,
									get = function(info) return mezdb.iconTargAllianceScale end,
									set = function(info,value) mezdb.iconTargAllianceScale = value 
										taiFrame:SetSize(mezdb.iconTargAllianceScale,mezdb.iconTargAllianceScale)
									end,
								},
								iconTargAllianceAlpha = {
									order = 5,
									type = "range",
									name = L["Icon Alpha"],
									desc = "Set desired transparency of icon",
									min = 0.1, max = 1, step = 0.1,
									get = function(info) return mezdb.iconTargAllianceAlpha end,
									set = function(info,value) mezdb.iconTargAllianceAlpha = value 
										taiTexture:SetAlpha(mezdb.iconTargAllianceAlpha)
									end,
								},
								iconTargAllianceStrata = {
									order = 6,
									type = "select",
									name = L["Strata"],
									desc = "Set if icon is in foreground or background etc",
									values = {
										['BACKGROUND'] = L["BACKGROUND"],
										['LOW'] = L["LOW"],
										['MEDIUM'] = L["MEDIUM"],
										['HIGH'] = L["HIGH"],
									},
									get = function(info) return mezdb.iconTargAllianceStrata end,
									set = function(info,value) mezdb.iconTargAllianceStrata = value; 
										taiFrame:SetFrameStrata(mezdb.iconTargAllianceStrata)
									end,
								},
							},
						},
						muiTargetH = {
							order = 2,
							type = "group",
							inline = true,
							name = "Horde",
							args = {
								iconTargHordeEnabled = {
									order = 1,
									type = "toggle",
									name = L["Enabled"],
									desc = "Enable/Disable target horde icon",
									get = function(info) return mezdb.iconTargHordeEnabled end,
									set = function(info,value) mezdb.iconTargHordeEnabled = value 
										muiUpdateTargetIcons()
									end,
								},
								iconTargHordeLock = {
									order = 2,
									type = "toggle",
									name = L["Locked"],
									desc = "Lock/unlock mouse drag movability",
									get = function(info) return mezdb.iconTargHordeLock end,
									set = function(info,value) mezdb.iconTargHordeLock = value;
										muiIconLockToggle()
									end,
								},
								blank3 = {
									type = "description",
									order = 3,
									name = " ",
								},
								iconTargHordeScale = {
									order = 4,
									type = "range",
									name = L["Icon Scale"],
									desc = "Set scale of icon",
									min = 30, max = 512, step = 1,
									get = function(info) return mezdb.iconTargHordeScale end,
									set = function(info,value) mezdb.iconTargHordeScale = value 
										thiFrame:SetSize(mezdb.iconTargHordeScale,mezdb.iconTargHordeScale)
									end,
								},
								iconTargHordeAlpha = {
									order = 5,
									type = "range",
									name = L["Icon Alpha"],
									desc = "Set desired transparency of icon",
									min = 0.1, max = 1, step = 0.1,
									get = function(info) return mezdb.iconTargHordeAlpha end,
									set = function(info,value) mezdb.iconTargHordeAlpha = value 
										thiTexture:SetAlpha(mezdb.iconTargHordeAlpha)
									end,
								},
								iconTargHordeStrata = {
									order = 6,
									type = "select",
									name = L["Strata"],
									desc = "Set if icon is in foreground or background etc",
									values = {
										['BACKGROUND'] = L["BACKGROUND"],
										['LOW'] = L["LOW"],
										['MEDIUM'] = L["MEDIUM"],
										['HIGH'] = L["HIGH"],
									},
									get = function(info) return mezdb.iconTargHordeStrata end,
									set = function(info,value) mezdb.iconTargHordeStrata = value; 
										thiFrame:SetFrameStrata(mezdb.iconTargHordeStrata)
									end,
								},
							},
						},
					},
				},
			},
		},
    },
}

function MezUI:OnInitialize()
    -- Called when the addon is loaded
	LibStub("AceConfig-3.0"):RegisterOptionsTable("MezUI", options)
	LibStub("LibAboutPanel").new(nil, "MezUI")
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("MezUI", options.args.muiHealthbarOverlays.name, "MezUI", "muiHealthbarOverlays")
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("MezUI", options.args.muiFactionIcons.name, "MezUI", "muiFactionIcons")
	
	--[[MezUI:RegisterChatCommand("mui", "ChatCommand")
	MezUI:RegisterChatCommand("MezUI", "ChatCommand")
    MezUI:RegisterChatCommand("MezUI", "ChatCommand")]]
end

local mezdbFrame = CreateFrame("Frame")
	mezdbFrame:RegisterEvent("PLAYER_LOGIN")
	mezdbFrame:SetScript("OnEvent", function(self, event)
	if event == "PLAYER_LOGIN" then
		MezDB = MezDB or {}
		mezdb = MezDB
		for var,value in pairs(defaults) do
			mezdb[var] = mezdb[var]==nil and value or mezdb[var]
		end
		
		-- Setup Player Faction Textures
		muiUpdatePlayerFaction()
		
		-- Setup Overlay
		mezdb.overlayLock = true
		muiOverlayFrameInit()
		if mezdb.overlayEnabled then
			poFrame:Show()
			toFrame:Hide()
		else
			poFrame:Hide()
			toFrame:Hide()
		end
		
		-- Setup Player Icon
		mezdb.iconPlayerLock = true
		muiPlayerIconInit()
		if not mezdb.iconPlayerEnabled then
			piFrame:Hide()
		end
		
		-- Setup Target Alliance Icon
		mezdb.iconTargAllianceLock = true
		muiTargetAllianceIconInit()
		if not mezdb.iconTargAllianceEnabled then
			taiFrame:Hide()
		end
		
		-- Setup Target Horde Icon
		mezdb.iconTargHordeLock = true
		muiTargetHordeIconInit()
		if not mezdb.iconTargHordeEnabled then
			thiFrame:Hide()
		end
	end
end)

--[[ If the Okay button is pressed in the options screen then check everything saved
	and do a reload. OR have the elements change when the options are changed without a reload
	this would probably be better way of doing it rather than requiring a reload
--]]

--[[ The PLAYER & TARGET overlay frames. These frames hold the background textures
	as well as the childframe for the embedded faction display if selected.
--]]

function muiOverlayFrameInit()
	-- //////// PLAYER
	poFrame:SetFrameStrata("MEDIUM")
	poFrame:SetSize(mezdb.overlayPlayerWidth,mezdb.overlayPlayerHeight)
	-- Setup the texture for the frame
	poTexture:SetTexture(mezdb.bgPlayerTexture)
	poTexture:SetAlpha(mezdb.bgPlayerAlpha)
	poTexture:SetAllPoints(poFrame)
	poFrame.texture = poTexture
	poFrame:ClearAllPoints()
	-- Setup the text for the frame when unlocked
	poFrame.text = poFrame:CreateFontString(nil,"ARTWORK") 
	poFrame.text:SetFont("Fonts\\FRIZQT__.TTF", 13, "OUTLINE")
	poFrame.text:SetPoint("CENTER",0,0)
	-- Positioning
	poFrame:SetPoint("BOTTOMLEFT", mezdb.overlayPlayerXpos, mezdb.overlayPlayerYpos)
	poFrame:Show()
	-- Make the player overlay frame movable.
	poFrame:SetMovable(true)
	poFrame:RegisterForDrag("LeftButton")
	poFrame:SetScript("OnDragStart", poFrame.StartMoving)
	-- Make the player overlay frame snap to the edges of the screen if moved past them.
	poFrame:SetScript("OnDragStop", function(self, event, ...)
		poFrame:StopMovingOrSizing()	
		-- Setup variables only available while this function executes
		local pox = poFrame:GetLeft()
		local poy = poFrame:GetBottom()
		
		-- Do snapping
		if (pox < 0) then
			pox = 0
		elseif (pox + poFrame:GetWidth() > GetScreenWidth()) then
			pox = GetScreenWidth() - poFrame:GetWidth()
		end
		if (poy < 0) then
			poy = 0
		elseif (poy + poFrame:GetHeight() > GetScreenHeight()) then
			poy = GetScreenHeight() - poFrame:GetHeight()
		end
		poFrame:ClearAllPoints()
		poFrame:SetPoint("BOTTOMLEFT", pox, poy)
		
		-- Save position
		mezdb.overlayPlayerXpos = pox
		mezdb.overlayPlayerYpos = poy
	end)
	--[[
	-- //////// PLAYER SCROLLCHILD
	poScrollChild:SetMovable(true)
	-- Setup the text for the frame when unlocked
	poScrollChild.text = poScrollChild:CreateFontString(nil,"ARTWORK") 
	poScrollChild.text:SetFont("Fonts\\FRIZQT__.TTF", 13, "OUTLINE")
	poScrollChild.text:SetPoint("CENTER",0,0)
	
	poScrollChild:RegisterForDrag("LeftButton")
	poScrollChild:SetScript("OnDragStart", poScrollChild.StartMoving)
	poScrollChild:SetScript("OnDragStop", function(self)
		self:StopMovingOrSizing()
	end)
	
	poScrollFrame:SetScrollChild(poScrollChild);
	poScrollFrame:SetAllPoints(poFrame);
	poScrollChild:SetSize(256, 256);
	--poChildTexture:SetAllPoints(poScrollChild)
	--poScrollChild.texture = poChildTexture
	
	poScrollChild:RegisterEvent("PLAYER_LOGIN")
	poScrollChild:SetScript("OnEvent", function(self, event)
		if event == "PLAYER_LOGIN" then
			local pofaction = UnitFactionGroup("player")
			if pofaction == "Alliance" then
				--poChildTexture:SetTexture(LSM:Fetch("background", "muiagrey"))
			elseif pofaction == "Horde" then
				--poChildTexture:SetTexture(LSM:Fetch("background", "muihgrey"))
			end
		end
	end)
]]
	-- //////// TARGET
	toFrame:SetFrameStrata("MEDIUM")
	toFrame:SetSize(mezdb.overlayTargWidth,mezdb.overlayTargHeight)
	-- Setup the texture for the frame.
	toTexture:SetTexture(mezdb.bgTargTexture) 
	toTexture:SetAlpha(mezdb.bgTargAlpha)
	toTexture:SetAllPoints(toFrame)
	toFrame.texture = toTexture
	toFrame:ClearAllPoints()
	-- Setup the text for the frame when unlocked
	toFrame.text = toFrame:CreateFontString(nil,"ARTWORK") 
	toFrame.text:SetFont("Fonts\\FRIZQT__.TTF", 13, "OUTLINE")
	toFrame.text:SetPoint("CENTER",0,0)
	-- Positioning
	toFrame:SetPoint("BOTTOMLEFT", mezdb.overlayTargXpos, mezdb.overlayTargYpos)
	toFrame:Hide()
	-- Make the target overlay frame movable.
	toFrame:SetMovable(true)
	toFrame:RegisterForDrag("LeftButton")
	toFrame:SetScript("OnDragStart", toFrame.StartMoving)
	-- Make the target overlay frame snap to the edges of the screen if moved past them.
	toFrame:SetScript("OnDragStop", function(self, event, ...)
		toFrame:StopMovingOrSizing()
		-- Setup variables only available while this function executes
		local tox = toFrame:GetLeft()
		local toy = toFrame:GetBottom()
		
		-- Do snapping
		if (tox < 0) then
			tox = 0
		elseif (tox + toFrame:GetWidth() > GetScreenWidth()) then
			tox = GetScreenWidth() - toFrame:GetWidth()
		end
		if (toy < 0) then
			toy = 0
		elseif (toy + toFrame:GetHeight() > GetScreenHeight()) then
			toy = GetScreenHeight() - toFrame:GetHeight()
		end
		toFrame:ClearAllPoints()
		toFrame:SetPoint("BOTTOMLEFT", tox, toy)
		
		-- Save position
		mezdb.overlayTargXpos = tox
		mezdb.overlayTargYpos = toy
	end)

	muiUpdateBG()
end



--[[
	Make a function to set frame to saved settings. This func will be called at end
	of frame creation in muiOverlayFrameInit as well as when resetting the locked function.
	Maybe move the frame moving out of the muiOverlayFrameInit function as well. However it currently
	still saves the position on a logout/login though it may make the code more clean.
	Could possibly use the same code for the different frames then by having a local
	variable within the frame being set to the frame depending on which option pressed.
--]]

function muiOverlayLockToggle()
	if mezdb.overlayEnabled then
		if mezdb.overlayLock then
			-- Player
			poFrame:EnableMouse(false)
			poFrame.text:SetText(nil)
			-- Target
			toFrame:EnableMouse(false)
			toFrame.text:SetText(nil)
			toFrame:Hide()
			
			muiUpdateBG()
		else
			-- Player
			poFrame:EnableMouse(true)
			poTexture:SetColorTexture(0,1,0,0.3);
			poFrame.text:SetText("Player")
			-- Target
			toFrame:EnableMouse(true)
			toTexture:SetColorTexture(0,0,1,0.6);
			toFrame.text:SetText("Target")
			toFrame:Show()
		end
	end
end

function muiIconLockToggle()
	-- Player
	if mezdb.iconPlayerEnabled then
		if mezdb.iconPlayerLock then
			piFrame:EnableMouse(false)
			piFrame.text:SetText(nil)
		else
			piFrame:EnableMouse(true)
			piFrame.text:SetText("Player")
		end
	end
	-- Target (Alliance)
	if mezdb.iconTargAllianceEnabled then
		if mezdb.iconTargAllianceLock then
			taiFrame:EnableMouse(false)
			taiFrame.text:SetText(nil)
			taiFrame:Hide()
		else
			taiFrame:EnableMouse(true)
			taiFrame.text:SetText("Target(A)")
			taiFrame:Show()
		end
	end
	-- Target (Horde)
	if mezdb.iconTargHordeEnabled then
		if mezdb.iconTargHordeLock then
			thiFrame:EnableMouse(false)
			thiFrame.text:SetText(nil)
			thiFrame:Hide()
		else
			thiFrame:EnableMouse(true)
			thiFrame.text:SetText("Target(H)")
			thiFrame:Show()
		end
	end
end
		
function muipoChildLockToggle()
	if mezdb.poChildLock then
		poScrollChild:EnableMouse(false)
		poScrollChild.text:SetText(nil)
	else
		poScrollChild:EnableMouse(true)
		poScrollChild.text:SetText("Unlocked")
	end
end

function muiUpdatePlayerFaction()
	local playerFaction = UnitFactionGroup("player")
	if playerFaction == "Alliance" then
		mezdb.bgPlayerTexture = mezdb.bgPlayerAllianceTexture
		mezdb.pIconTexture = mezdb.iconPlayerAllianceTexture
	elseif playerFaction == "Horde" then
		mezdb.bgPlayerTexture = mezdb.bgPlayerHordeTexture
		mezdb.pIconTexture = mezdb.iconPlayerHordeTexture
	end
end

function muiUpdateBG()	
	muiUpdatePlayerFaction()
	-- Player
	poTexture:SetAlpha(mezdb.bgPlayerAlpha) -- reset alpha to chosen alpha in settings
	poTexture:SetTexture(LSM:Fetch("statusbar", mezdb.bgPlayerTexture))
	-- Target
	muiUpdateTargetOverlays()
	
	if mezdb.overlayEnabled then
		poFrame:Show()
	else
		poFrame:Hide()
		toFrame:Hide()
	end
	-- reset alpha to chosen alpha in settings
	--toTexture:SetTexture(LSM:Fetch("statusbar", mezdb.bgTargTexture))
end

function muiUpdatePlayerIcon()
	if mezdb.iconPlayerEnabled then
		piFrame:Show()
		piTexture:SetAlpha(mezdb.iconPlayerAlpha)
		piTexture:SetTexture(LSM:Fetch("background", mezdb.pIconTexture))
	else
		piFrame:Hide()
	end
end

toFrame:RegisterEvent("PLAYER_TARGET_CHANGED")
toFrame:SetScript("OnEvent", function(self, event)
	if event == "PLAYER_TARGET_CHANGED" then
		muiUpdateTargetOverlays()
		muiUpdateTargetIcons()
	end
end)

function muiUpdateTargetOverlays()
	if (UnitExists("target")) and mezdb.overlayEnabled then
		local targetFaction = UnitFactionGroup("target")
		if targetFaction == "Alliance" then
			mezdb.bgTargTexture = mezdb.bgTargAllianceTexture
			toTexture:SetTexture(LSM:Fetch("statusbar", mezdb.bgTargAllianceTexture))
		elseif targetFaction == "Horde" then
			mezdb.bgTargTexture = mezdb.bgTargHordeTexture
			toTexture:SetTexture(LSM:Fetch("statusbar", mezdb.bgTargHordeTexture))
		else
			mezdb.bgTargTexture = mezdb.bgTargNeutralTexture
			toTexture:SetTexture(LSM:Fetch("statusbar", mezdb.bgTargNeutralTexture))
		end
		toFrame:Show()
	else
		toFrame:Hide()
	end
	toTexture:SetAlpha(mezdb.bgTargAlpha)
end

function muiUpdateTargetIcons()
	if (UnitExists("target")) then
		local targetFaction = UnitFactionGroup("target")
		if targetFaction == "Alliance" and mezdb.iconTargAllianceEnabled then
			taiTexture:SetAlpha(mezdb.iconTargAllianceAlpha)
			taiTexture:SetTexture(LSM:Fetch("background", mezdb.iconTargAllianceTexture))
			taiFrame:Show()
			thiFrame:Hide()
		elseif targetFaction == "Horde" and mezdb.iconTargHordeEnabled then
			thiTexture:SetAlpha(mezdb.iconTargHordeAlpha)
			thiTexture:SetTexture(LSM:Fetch("background", mezdb.iconTargHordeTexture))
			taiFrame:Hide()
			thiFrame:Show()
		else
			taiFrame:Hide()
			thiFrame:Hide()
		end
	else
		taiFrame:Hide()
		thiFrame:Hide()
	end
	if mezdb.iconTargAllianceEnabled and not mezdb.iconTargAllianceLock then
		taiFrame:Show()
	end
	if mezdb.iconTargHordeEnabled and not mezdb.iconTargHordeLock then
		thiFrame:Show()
	end
end

function muiPlayerIconInit()
	piFrame:SetFrameStrata(mezdb.iconPlayerStrata)
	piFrame:SetSize(mezdb.iconPlayerScale,mezdb.iconPlayerScale)
	-- Setup the texture for the frame.
	piTexture:SetTexture(mezdb.pIconTexture) 
	piTexture:SetAlpha(mezdb.iconPlayerAlpha)
	piTexture:SetAllPoints(piFrame)
	piFrame.texture = piTexture
	piFrame:ClearAllPoints()
	-- Setup the text for the frame when unlocked
	piFrame.text = piFrame:CreateFontString(nil,"ARTWORK") 
	piFrame.text:SetFont("Fonts\\FRIZQT__.TTF", 13, "OUTLINE")
	piFrame.text:SetPoint("CENTER",0,0)
	-- Positioning
	piFrame:SetPoint("BOTTOMLEFT", mezdb.iconPlayerXpos, mezdb.iconPlayerYpos)
	piFrame:Show()
	-- Make the target overlay frame movable.
	piFrame:SetMovable(true)
	piFrame:RegisterForDrag("LeftButton")
	piFrame:SetScript("OnDragStart", piFrame.StartMoving)
	-- Make the target overlay frame snap to the edges of the screen if moved past them.
	piFrame:SetScript("OnDragStop", function(self, event, ...)
		piFrame:StopMovingOrSizing()
		-- Setup variables only available while this function executes
		local pix = piFrame:GetLeft()
		local piy = piFrame:GetBottom()
		
		-- Do snapping
		if (pix < 0) then
			pix = 0
		elseif (pix + piFrame:GetWidth() > GetScreenWidth()) then
			pix = GetScreenWidth() - piFrame:GetWidth()
		end
		if (piy < 0) then
			piy = 0
		elseif (piy + piFrame:GetHeight() > GetScreenHeight()) then
			piy = GetScreenHeight() - piFrame:GetHeight()
		end
		piFrame:ClearAllPoints()
		piFrame:SetPoint("BOTTOMLEFT", pix, piy)
		
		-- Save position
		mezdb.iconPlayerXpos = pix
		mezdb.iconPlayerYpos = piy
	end)
	muiUpdatePlayerIcon()
end

function muiTargetAllianceIconInit()
	taiFrame:SetFrameStrata(mezdb.iconTargAllianceStrata)
	taiFrame:SetSize(mezdb.iconTargAllianceScale,mezdb.iconTargAllianceScale)
	-- Setup the texture for the frame.
	taiTexture:SetTexture(LSM:Fetch("background", mezdb.iconTargAllianceTexture))
	taiTexture:SetAlpha(mezdb.iconTargAllianceAlpha)
	taiTexture:SetAllPoints(taiFrame)
	taiFrame.texture = taiTexture
	taiFrame:ClearAllPoints()
	-- Setup the text for the frame when unlocked
	taiFrame.text = taiFrame:CreateFontString(nil,"ARTWORK") 
	taiFrame.text:SetFont("Fonts\\FRIZQT__.TTF", 13, "OUTLINE")
	taiFrame.text:SetPoint("CENTER",0,0)
	-- Positioning
	taiFrame:SetPoint("BOTTOMLEFT", mezdb.iconTargAllianceXpos, mezdb.iconTargAllianceYpos)
	taiFrame:Hide()
	-- Make the target overlay frame movable.
	taiFrame:SetMovable(true)
	taiFrame:RegisterForDrag("LeftButton")
	taiFrame:SetScript("OnDragStart", taiFrame.StartMoving)
	-- Make the target overlay frame snap to the edges of the screen if moved past them.
	taiFrame:SetScript("OnDragStop", function(self, event, ...)
		taiFrame:StopMovingOrSizing()
		-- Setup variables only available while this function executes
		local taix = taiFrame:GetLeft()
		local taiy = taiFrame:GetBottom()
		
		-- Do snapping
		if (taix < 0) then
			taix = 0
		elseif (taix + taiFrame:GetWidth() > GetScreenWidth()) then
			taix = GetScreenWidth() - taiFrame:GetWidth()
		end
		if (taiy < 0) then
			taiy = 0
		elseif (taiy + taiFrame:GetHeight() > GetScreenHeight()) then
			taiy = GetScreenHeight() - taiFrame:GetHeight()
		end
		taiFrame:ClearAllPoints()
		taiFrame:SetPoint("BOTTOMLEFT", taix, taiy)
		
		-- Save position
		mezdb.iconTargAllianceXpos = taix
		mezdb.iconTargAllianceYpos = taiy
	end)
	muiUpdateTargetIcons()
end

function muiTargetHordeIconInit()
	thiFrame:SetFrameStrata(mezdb.iconTargHordeStrata)
	thiFrame:SetSize(mezdb.iconTargHordeScale,mezdb.iconTargHordeScale)
	-- Setup the texture for the frame.
	thiTexture:SetTexture(LSM:Fetch("background", mezdb.iconTargHordeTexture))
	thiTexture:SetAlpha(mezdb.iconTargHordeAlpha)
	thiTexture:SetAllPoints(thiFrame)
	thiFrame.texture = thiTexture
	thiFrame:ClearAllPoints()
	-- Setup the text for the frame when unlocked
	thiFrame.text = thiFrame:CreateFontString(nil,"ARTWORK") 
	thiFrame.text:SetFont("Fonts\\FRIZQT__.TTF", 13, "OUTLINE")
	thiFrame.text:SetPoint("CENTER",0,0)
	-- Positioning
	thiFrame:SetPoint("BOTTOMLEFT", mezdb.iconTargHordeXpos, mezdb.iconTargHordeYpos)
	thiFrame:Hide()
	-- Make the target overlay frame movable.
	thiFrame:SetMovable(true)
	thiFrame:RegisterForDrag("LeftButton")
	thiFrame:SetScript("OnDragStart", thiFrame.StartMoving)
	-- Make the target overlay frame snap to the edges of the screen if moved past them.
	thiFrame:SetScript("OnDragStop", function(self, event, ...)
		thiFrame:StopMovingOrSizing()
		-- Setup variables only available while this function executes
		local thix = thiFrame:GetLeft()
		local thiy = thiFrame:GetBottom()
		
		-- Do snapping
		if (thix < 0) then
			thix = 0
		elseif (thix + thiFrame:GetWidth() > GetScreenWidth()) then
			thix = GetScreenWidth() - thiFrame:GetWidth()
		end
		if (thiy < 0) then
			thiy = 0
		elseif (thiy + thiFrame:GetHeight() > GetScreenHeight()) then
			thiy = GetScreenHeight() - thiFrame:GetHeight()
		end
		thiFrame:ClearAllPoints()
		thiFrame:SetPoint("BOTTOMLEFT", thix, thiy)
		
		-- Save position
		mezdb.iconTargHordeXpos = thix
		mezdb.iconTargHordeYpos = thiy
	end)
	muiUpdateTargetIcons()
end

E.UIParent:HookScript("OnHide", function()
	poFrame:Hide()
	toFrame:Hide()
	piFrame:Hide()
	taiFrame:Hide()
	thiFrame:Hide()
end)

E.UIParent:HookScript("OnShow", function()
	muiUpdateBG()
	muiUpdateTargetOverlays()
	muiUpdatePlayerIcon()
	muiUpdateTargetIcons()
end)

function muiElvUIConfig()
	if mezdb.overlayEnabled then
		-- Player Overlay
		poFrame:SetSize(ElvUF_Player.Health:GetWidth(),ElvUF_Player.Health:GetHeight())
		poFrame:SetPoint("BOTTOMLEFT", ElvUF_Player.Health:GetLeft(), ElvUF_Player.Health:GetBottom());
		
		-- Target Overlay
		toFrame:SetSize(ElvUF_Target.Health:GetWidth(),ElvUF_Target.Health:GetHeight())
		toFrame:SetPoint("BOTTOMLEFT", ElvUF_Target.Health:GetLeft(), ElvUF_Target.Health:GetBottom());
		
		-- Save Position and Size
		mezdb.overlayPlayerXpos = poFrame:GetLeft()
		mezdb.overlayPlayerYpos = poFrame:GetBottom()
		mezdb.overlayPlayerWidth = poFrame:GetWidth()
		mezdb.overlayPlayerHeight = poFrame:GetHeight()
		mezdb.overlayTargXpos = toFrame:GetLeft()
		mezdb.overlayTargYpos = toFrame:GetBottom()
		mezdb.overlayTargWidth = toFrame:GetWidth()
		mezdb.overlayTargHeight = toFrame:GetHeight()
	end
end